import pandas as pd
import numpy as np
from scipy import stats


def edit_distance(ref, alt):
    assert len(ref)==2
    assert len(alt)==2
    
    if alt=='NN':
        # return(np.random.randint(3))
        return(0)
    
    return np.sum([ref[i]!=alt[i] for i in range(2)])

def get_matrices(feat_file, label_file, kinship_file):
    feats = pd.read_csv(feat_file, sep='\t')
    labels = pd.read_csv(label_file, sep='\t')
    Kinship = pd.read_csv(kinship_file, header=None)
    
    samples = list(set(Kinship[0]) & set(feats.columns[11:]) & set(labels['SampleID']))
    
    cat_mat = feats[samples].values
    num_mat = np.zeros(cat_mat.shape)
    for i in range(num_mat.shape[0]):
        homo_gp = [gg for gg in cat_mat[i] if gg[0]==gg[1]]
        ref, _ = stats.mode(homo_gp)
        ref = ref[0]
        assert ref!='NN' 
        if ref[0]!=ref[1]:
            print(ref, cat_mat[i])
        for j in range(cat_mat.shape[1]):
            num_mat[i][j] = edit_distance(ref, cat_mat[i][j])   
            
    X = num_mat.T
    
    Y = labels.set_index('SampleID').loc[samples].values
    Kinship_mapping = {col: i for i, col in enumerate(Kinship[0])}
    selected_kinship_cols = [Kinship_mapping[sample_id] for sample_id in samples]
    K = (Kinship[Kinship.columns[1:]].values)[selected_kinship_cols, :][:,selected_kinship_cols]
    
    return X, Y, K

def get_sorghum_matrices(feat_file='GBS_Markers_Sorghum_116128_n=320_Inds.txt1.hmp.txt', 
                         label_file='BrownSAP2008_Inflorescence_for_TASSELGBS_NAMES.txt', 
                         kinship_file='GAPIT.Kin.Loiselle.csv'):
    
    feats = pd.read_csv(feat_file, sep='\t')
    labels = pd.read_csv(label_file, sep='\t', index_col=0)
    Kinship = pd.read_csv(kinship_file, header=None, index_col=0)
    
    K = Kinship.values
    samples = Kinship.index
    
    num_mat = np.zeros(feats[samples].values.shape)
    for i, row in feats.iterrows():
        ref, minor = row['alleles'].split('/')
        # num_mat[i, :] = (row[samples]!=ref)
        for j, geno in enumerate(row[samples]):
            if geno==ref:
                num_mat[i, j] = 0
            elif geno==minor:
                num_mat[i, j] = 2
            else:
                num_mat[i, j] = 1
    X = num_mat.T
    
    Y = labels.loc[samples].values
    
    return X, Y, K
    
    
    
    
    
    